@@ -1,6 +1,6 @@
 from typing import Union, List
 
-from .base import CompileTimeValue
+from ._base import CompileTimeValue
 from .nbt import CplNBT
 from ..tokenizer import Token
 from ..utils import INT_TYPE
@@ -15,19 +15,13 @@ class CplIntNBT(CplNBT):
             ctx.file.append(f"execute store result score {score_loc} run data get {self.location}")
             t = "int"
             if force_t == "float":
-                ctx.file.append(f"scoreboard players operation {score_loc} *= FLOAT_PREC --temp--")
+                ctx.file.append(f"scoreboard players operation {score_loc} *= FLOAT_PREC __temp__")
                 t = "float"
             return CplScore(self.token, score_loc, t)
         return super()._cache(ctx, score_loc, nbt_loc, force, force_t)
 
-    def _get_index(self, ctx, index: CompileTimeValue):
-        return None
-
-    def _get_slice(self, ctx, index1, index2, index3):
-        return None
-
-    def _call_index(self, ctx, index: str, arguments: List[CompileTimeValue]):
-        return self.cache(ctx, force="score").call_index(ctx, index, arguments)
+    def _call_index(self, ctx, index: str, arguments: List[CompileTimeValue], token):
+        return self.cache(ctx, force="score")._call_index(ctx, index, arguments, token)
 
 
 from .score import CplScore
